@@ -53,6 +53,8 @@
 #include "ui_textareaappearanceconfigwidget.h"
 
 #include <KIO/Job>
+#include <KIO/OpenUrlJob>
+#include <KIO/JobUiDelegate>
 
 #include "kateabstractinputmodefactory.h"
 #include "katepartdebug.h"
@@ -63,7 +65,6 @@
 #include <KLineEdit>
 #include <KMessageBox>
 #include <KProcess>
-#include <KRun>
 #include <KSeparator>
 #include <KShortcutsDialog>
 
@@ -883,11 +884,7 @@ KateSaveConfigTab::KateSaveConfigTab(QWidget *parent)
     observeChanges(uiadv->chkBackupLocalFiles);
     observeChanges(uiadv->chkBackupRemoteFiles);
     observeChanges(uiadv->cmbSwapFileMode);
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
     connect(uiadv->cmbSwapFileMode, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &KateSaveConfigTab::swapFileModeChanged);
-#else
-    connect(uiadv->cmbSwapFileMode, QOverload<int, const QString &>::of(&QComboBox::currentIndexChanged), this, &KateSaveConfigTab::swapFileModeChanged);
-#endif
 
 
     observeChanges(uiadv->edtBackupPrefix);
@@ -1123,7 +1120,6 @@ KateGotoBar::KateGotoBar(KTextEditor::View *view, QWidget *parent)
     topLayout->addWidget(btn);
 
     m_gotoRange = new QSpinBox(this);
-    m_gotoRange->setMinimum(1);
     topLayout->addWidget(m_gotoRange, 1);
     topLayout->setStretchFactor(m_gotoRange, 0);
 
@@ -1200,9 +1196,9 @@ bool KateGotoBar::eventFilter(QObject *object, QEvent *event)
 
 void KateGotoBar::gotoClipboard()
 {
-    QRegularExpression rx(QStringLiteral("\\d+"));
+    QRegularExpression rx(QStringLiteral("-?\\d+"));
     int lineNo = rx.match(QApplication::clipboard()->text(QClipboard::Selection)).captured().toInt();
-    if (lineNo <= m_gotoRange->maximum() && lineNo >= 1) {
+    if (lineNo >= m_gotoRange->minimum() && lineNo <= m_gotoRange->maximum()) {
         m_gotoRange->setValue(lineNo);
         gotoLine();
     } else {
@@ -1216,7 +1212,9 @@ void KateGotoBar::gotoClipboard()
 
 void KateGotoBar::updateData()
 {
-    m_gotoRange->setMaximum(m_view->document()->lines());
+    int lines = m_view->document()->lines();
+    m_gotoRange->setMinimum(-lines);
+    m_gotoRange->setMaximum(lines);
     if (!isVisible()) {
         m_gotoRange->setValue(m_view->cursorPosition().line() + 1);
         m_gotoRange->adjustSize(); // ### does not respect the range :-(
@@ -1242,7 +1240,14 @@ void KateGotoBar::gotoLine()
         kv->clearSelection();
     }
 
-    m_view->setCursorPosition(KTextEditor::Cursor(m_gotoRange->value() - 1, 0));
+    int gotoValue = m_gotoRange->value();
+    if (gotoValue < 0) {
+        gotoValue += m_view->document()->lines();
+    } else if (gotoValue > 0) {
+        gotoValue -= 1;
+    }
+
+    m_view->setCursorPosition(KTextEditor::Cursor(gotoValue, 0));
     m_view->setFocus();
     emit hideMe();
 }
@@ -1321,6 +1326,7 @@ KateModOnHdPrompt::KateModOnHdPrompt(KTextEditor::DocumentPrivate *doc, KTextEdi
 
         if (!QStandardPaths::findExecutable(QStringLiteral("diff")).isEmpty()) {
             m_diffAction = new QAction(i18n("View &Difference"), this);
+            m_diffAction->setIcon(QIcon::fromTheme(QStringLiteral("document-multiple")));
             m_diffAction->setToolTip(i18n("Shows a diff of the changes"));
             m_message->addAction(m_diffAction, false);
             connect(m_diffAction, SIGNAL(triggered()), this, SLOT(slotDiff()));
@@ -1347,7 +1353,7 @@ KateModOnHdPrompt::KateModOnHdPrompt(KTextEditor::DocumentPrivate *doc, KTextEdi
 
     QAction *aIgnore = new QAction(i18n("&Ignore"), this);
     aIgnore->setToolTip(i18n("Ignores the changes on disk without any action."));
-    aIgnore->setIcon(KStandardGuiItem::overwrite().icon());
+    aIgnore->setIcon(QIcon::fromTheme(QStringLiteral("dialog-cancel")));
     m_message->addAction(aIgnore);
     connect(aIgnore, SIGNAL(triggered()), this, SIGNAL(ignoreTriggered()));
 
@@ -1372,7 +1378,7 @@ void KateModOnHdPrompt::slotDiff()
         return;
     }
 
-    m_diffFile = new QTemporaryFile();
+    m_diffFile = new QTemporaryFile(QLatin1String("XXXXXX.diff"));
     m_diffFile->open();
 
     // Start a KProcess that creates a diff
@@ -1432,8 +1438,10 @@ void KateModOnHdPrompt::slotPDone()
     delete m_diffFile;
     m_diffFile = nullptr;
 
-    // KRun::runUrl should delete the file, once the client exits
-    KRun::runUrl(url, QStringLiteral("text/x-patch"), nullptr, KRun::RunFlags(KRun::DeleteTemporaryFiles));
+    KIO::OpenUrlJob *job = new KIO::OpenUrlJob(url, QStringLiteral("text/x-patch"));
+    job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, nullptr /*TODO window*/));
+    job->setDeleteTemporaryFile(true); // delete the file, once the client exits
+    job->start();
 }
 
 // END KateModOnHdPrompt
