@@ -1,60 +1,61 @@
-/*  This file is part of the KDE libraries and the Kate part.
- *
- *  Copyright (C) 2001-2010 Christoph Cullmann <cullmann@kde.org>
- *  Copyright (C) 2009 Erlend Hamberg <ehamberg@gmail.com>
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
- */
+/*  SPDX-License-Identifier: LGPL-2.0-or-later
+
+    Copyright (C) 2001-2010 Christoph Cullmann <cullmann@kde.org>
+    Copyright (C) 2009 Erlend Hamberg <ehamberg@gmail.com>
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Library General Public License for more details.
+
+    You should have received a copy of the GNU Library General Public License
+    along with this library; see the file COPYING.LIB.  If not, write to
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
+*/
 
 #include "kateglobal.h"
 #include "config.h"
 
 #include <ktexteditor_version.h>
 
-#include "katedocument.h"
-#include "kateview.h"
-#include "katerenderer.h"
+#include "katebuffer.h"
 #include "katecmd.h"
 #include "katecmds.h"
-#include "katesedcmd.h"
+#include "kateconfig.h"
+#include "katedefaultcolors.h"
+#include "katedocument.h"
 #include "katehighlightingcmds.h"
+#include "katekeywordcompletion.h"
 #include "katemodemanager.h"
+#include "katepartdebug.h"
+#include "katerenderer.h"
 #include "kateschema.h"
 #include "kateschemaconfig.h"
-#include "kateconfig.h"
 #include "katescriptmanager.h"
-#include "katebuffer.h"
+#include "katesedcmd.h"
+#include "katevariableexpansionmanager.h"
+#include "kateview.h"
 #include "katewordcompletion.h"
-#include "katekeywordcompletion.h"
 #include "spellcheck/spellcheck.h"
-#include "katepartdebug.h"
-#include "katedefaultcolors.h"
 
 #include "katenormalinputmodefactory.h"
 #include "kateviinputmodefactory.h"
 
-#include <kdirwatch.h>
-#include <KLocalizedString>
 #include <KAboutData>
+#include <KConfigGroup>
+#include <KLocalizedString>
 #include <KPageDialog>
 #include <KPageWidgetModel>
-#include <KConfigGroup>
+#include <KDirWatch>
 
-#include <QBoxLayout>
 #include <QApplication>
+#include <QBoxLayout>
 #include <QClipboard>
 #include <QPushButton>
 #include <QStringListModel>
@@ -63,10 +64,7 @@
 #include <git2.h>
 #endif
 
-// logging category for this framework, default: log stuff >= warning
-Q_LOGGING_CATEGORY(LOG_KTE, "org.kde.ktexteditor", QtWarningMsg)
-
-//BEGIN unit test mode
+// BEGIN unit test mode
 static bool kateUnitTestMode = false;
 
 void KTextEditor::EditorPrivate::enableUnitTestMode()
@@ -78,13 +76,18 @@ bool KTextEditor::EditorPrivate::unitTestMode()
 {
     return kateUnitTestMode;
 }
-//END unit test mode
+// END unit test mode
 
 KTextEditor::EditorPrivate::EditorPrivate(QPointer<KTextEditor::EditorPrivate> &staticInstance)
-    : KTextEditor::Editor (this)
-    , m_aboutData(QStringLiteral("katepart"), i18n("Kate Part"), QStringLiteral(KTEXTEDITOR_VERSION_STRING),
-                  i18n("Embeddable editor component"), KAboutLicense::LGPL_V2,
-                  i18n("(c) 2000-2017 The Kate Authors"), QString(), QStringLiteral("http://kate-editor.org"))
+    : KTextEditor::Editor(this)
+    , m_aboutData(QStringLiteral("katepart"),
+                  i18n("Kate Part"),
+                  QStringLiteral(KTEXTEDITOR_VERSION_STRING),
+                  i18n("Embeddable editor component"),
+                  KAboutLicense::LGPL_V2,
+                  i18n("(c) 2000-2019 The Kate Authors"),
+                  QString(),
+                  QStringLiteral("https://kate-editor.org"))
     , m_dummyApplication(nullptr)
     , m_application(&m_dummyApplication)
     , m_dummyMainWindow(nullptr)
@@ -95,16 +98,6 @@ KTextEditor::EditorPrivate::EditorPrivate(QPointer<KTextEditor::EditorPrivate> &
     // remember this
     staticInstance = this;
 
-#if QT_VERSION < QT_VERSION_CHECK(5, 9, 1)
-    // disable the QML JIT compiler as a protection against an unknown bug
-    // in Qt's V4 engine which can provoke a crash in certain of our scripts.
-    // See https://bugreports.qt.io/browse/QTBUG-63045
-    // https://bugs.kde.org/show_bug.cgi?id=384404
-    // and https://bugs.kde.org/show_bug.cgi?id=385413
-    qputenv("QV4_FORCE_INTERPRETER", QByteArrayLiteral("1"));
-    qCDebug(LOG_KTE) << "QV4_FORCE_INTERPRETER set to 1";
-#endif
-
     // init libgit2, we require at least 0.22 which has this function!
 #if LIBGIT2_FOUND
     git_libgit2_init();
@@ -120,13 +113,13 @@ KTextEditor::EditorPrivate::EditorPrivate(QPointer<KTextEditor::EditorPrivate> &
     //
     // fill about data
     //
-    m_aboutData.addAuthor(i18n("Christoph Cullmann"), i18n("Maintainer"), QStringLiteral("cullmann@kde.org"), QStringLiteral("http://www.cullmann.io"));
+    m_aboutData.addAuthor(i18n("Christoph Cullmann"), i18n("Maintainer"), QStringLiteral("cullmann@kde.org"), QStringLiteral("https://cullmann.io"));
     m_aboutData.addAuthor(i18n("Dominik Haumann"), i18n("Core Developer"), QStringLiteral("dhaumann@kde.org"));
-    m_aboutData.addAuthor(i18n("Milian Wolff"), i18n("Core Developer"), QStringLiteral("mail@milianw.de"), QStringLiteral("http://milianw.de"));
+    m_aboutData.addAuthor(i18n("Milian Wolff"), i18n("Core Developer"), QStringLiteral("mail@milianw.de"), QStringLiteral("https://milianw.de/"));
     m_aboutData.addAuthor(i18n("Joseph Wenninger"), i18n("Core Developer"), QStringLiteral("jowenn@kde.org"), QStringLiteral("http://stud3.tuwien.ac.at/~e9925371"));
-    m_aboutData.addAuthor(i18n("Erlend Hamberg"), i18n("Vi Input Mode"), QStringLiteral("ehamberg@gmail.com"), QStringLiteral("http://hamberg.no/erlend"));
+    m_aboutData.addAuthor(i18n("Erlend Hamberg"), i18n("Vi Input Mode"), QStringLiteral("ehamberg@gmail.com"), QStringLiteral("https://hamberg.no/erlend"));
     m_aboutData.addAuthor(i18n("Bernhard Beschow"), i18n("Developer"), QStringLiteral("bbeschow@cs.tu-berlin.de"), QStringLiteral("https://user.cs.tu-berlin.de/~bbeschow"));
-    m_aboutData.addAuthor(i18n("Anders Lund"), i18n("Core Developer"), QStringLiteral("anders@alweb.dk"), QStringLiteral("http://www.alweb.dk"));
+    m_aboutData.addAuthor(i18n("Anders Lund"), i18n("Core Developer"), QStringLiteral("anders@alweb.dk"), QStringLiteral("https://alweb.dk"));
     m_aboutData.addAuthor(i18n("Michel Ludwig"), i18n("On-the-fly spell checking"), QStringLiteral("michel.ludwig@kdemail.net"));
     m_aboutData.addAuthor(i18n("Pascal Létourneau"), i18n("Large scale bug fixing"), QStringLiteral("pascal.letourneau@gmail.com"));
     m_aboutData.addAuthor(i18n("Hamish Rodda"), i18n("Core Developer"), QStringLiteral("rodda@kde.org"));
@@ -142,9 +135,9 @@ KTextEditor::EditorPrivate::EditorPrivate(QPointer<KTextEditor::EditorPrivate> &
     m_aboutData.addAuthor(i18n("Scott Manson"), i18n("KWrite XML Syntax highlighting support"), QStringLiteral("sdmanson@alltel.net"));
     m_aboutData.addAuthor(i18n("John Firebaugh"), i18n("Patches and more"), QStringLiteral("jfirebaugh@kde.org"));
     m_aboutData.addAuthor(i18n("Andreas Kling"), i18n("Developer"), QStringLiteral("kling@impul.se"));
-    m_aboutData.addAuthor(i18n("Mirko Stocker"), i18n("Various bugfixes"), QStringLiteral("me@misto.ch"), QStringLiteral("http://misto.ch/"));
+    m_aboutData.addAuthor(i18n("Mirko Stocker"), i18n("Various bugfixes"), QStringLiteral("me@misto.ch"), QStringLiteral("https://misto.ch/"));
     m_aboutData.addAuthor(i18n("Matthew Woehlke"), i18n("Selection, KColorScheme integration"), QStringLiteral("mw_triad@users.sourceforge.net"));
-    m_aboutData.addAuthor(i18n("Sebastian Pipping"), i18n("Search bar back- and front-end"), QStringLiteral("webmaster@hartwork.org"), QStringLiteral("http://www.hartwork.org/"));
+    m_aboutData.addAuthor(i18n("Sebastian Pipping"), i18n("Search bar back- and front-end"), QStringLiteral("webmaster@hartwork.org"), QStringLiteral("https://hartwork.org/"));
     m_aboutData.addAuthor(i18n("Jochen Wilhelmy"), i18n("Original KWrite Author"), QStringLiteral("digisnap@cs.tu-berlin.de"));
     m_aboutData.addAuthor(i18n("Gerald Senarclens de Grancy"), i18n("QA and Scripting"), QStringLiteral("oss@senarclens.eu"), QStringLiteral("http://find-santa.eu/"));
 
@@ -167,9 +160,9 @@ KTextEditor::EditorPrivate::EditorPrivate(QPointer<KTextEditor::EditorPrivate> &
     m_aboutData.setTranslator(i18nc("NAME OF TRANSLATORS", "Your names"), i18nc("EMAIL OF TRANSLATORS", "Your emails"));
 
     /**
-     * set the new Kate mascot
+     * set proper Kate icon for our about dialog
      */
-    m_aboutData.setProgramLogo (QImage(QStringLiteral(":/ktexteditor/mascot.png")));
+    m_aboutData.setProgramLogo(QIcon(QStringLiteral(":/ktexteditor/kate.svg")));
 
     //
     // dir watch
@@ -181,6 +174,11 @@ KTextEditor::EditorPrivate::EditorPrivate(QPointer<KTextEditor::EditorPrivate> &
     //
     m_cmdManager = new KateCmd();
 
+    //
+    // variable expansion manager
+    //
+    m_variableExpansionManager = new KateVariableExpansionManager(this);
+
     //
     // hl manager
     //
@@ -235,7 +233,7 @@ KTextEditor::EditorPrivate::EditorPrivate(QPointer<KTextEditor::EditorPrivate> &
     m_wordCompletionModel = new KateWordCompletionModel(this);
 
     // global keyword completion model
-    m_keywordCompletionModel = new KateKeywordCompletionModel (this);
+    m_keywordCompletionModel = new KateKeywordCompletionModel(this);
 
     // tap to QApplication object for color palette changes
     qApp->installEventFilter(this);
@@ -262,6 +260,10 @@ KTextEditor::EditorPrivate::~EditorPrivate()
     // cu model
     delete m_wordCompletionModel;
 
+    // delete variable expansion manager
+    delete m_variableExpansionManager;
+    m_variableExpansionManager = nullptr;
+
     // delete the commands before we delete the cmd manager
     qDeleteAll(m_cmds);
     delete m_cmdManager;
@@ -283,7 +285,7 @@ KTextEditor::Document *KTextEditor::EditorPrivate::createDocument(QObject *paren
     return doc;
 }
 
-//END KTextEditor::Editor config stuff
+// END KTextEditor::Editor config stuff
 
 void KTextEditor::EditorPrivate::configDialog(QWidget *parent)
 {
@@ -304,7 +306,7 @@ void KTextEditor::EditorPrivate::configDialog(QWidget *parent)
         item->setIcon(cp->icon());
 
         QVBoxLayout *topLayout = new QVBoxLayout(page);
-        topLayout->setMargin(0);
+        topLayout->setContentsMargins(0, 0, 0, 0);
 
         connect(kd->button(QDialogButtonBox::Apply), SIGNAL(clicked()), cp, SLOT(apply()));
         topLayout->addWidget(cp);
@@ -407,25 +409,25 @@ KTextEditor::EditorPrivate *KTextEditor::EditorPrivate::self()
 
 void KTextEditor::EditorPrivate::registerDocument(KTextEditor::DocumentPrivate *doc)
 {
-    Q_ASSERT (!m_documents.contains(doc));
+    Q_ASSERT(!m_documents.contains(doc));
     m_documents.insert(doc, doc);
 }
 
 void KTextEditor::EditorPrivate::deregisterDocument(KTextEditor::DocumentPrivate *doc)
 {
-    Q_ASSERT (m_documents.contains(doc));
+    Q_ASSERT(m_documents.contains(doc));
     m_documents.remove(doc);
 }
 
 void KTextEditor::EditorPrivate::registerView(KTextEditor::ViewPrivate *view)
 {
-    Q_ASSERT (!m_views.contains(view));
+    Q_ASSERT(!m_views.contains(view));
     m_views.insert(view);
 }
 
 void KTextEditor::EditorPrivate::deregisterView(KTextEditor::ViewPrivate *view)
 {
-    Q_ASSERT (m_views.contains(view));
+    Q_ASSERT(m_views.contains(view));
     m_views.remove(view);
 }
 
@@ -444,6 +446,11 @@ QStringList KTextEditor::EditorPrivate::commandList() const
     return m_cmdManager->commandList();
 }
 
+KateVariableExpansionManager *KTextEditor::EditorPrivate::variableExpansionManager()
+{
+    return m_variableExpansionManager;
+}
+
 void KTextEditor::EditorPrivate::updateColorPalette()
 {
     // update default color cache
@@ -471,9 +478,10 @@ void KTextEditor::EditorPrivate::copyToClipboard(const QString &text)
     QApplication::clipboard()->setText(text, QClipboard::Clipboard);
 
     /**
-     * remember in history
+     * LRU, kill potential duplicated, move new entry to top
      * cut after 10 entries
      */
+    m_clipboardHistory.removeOne(text);
     m_clipboardHistory.prepend(text);
     if (m_clipboardHistory.size() > 10) {
         m_clipboardHistory.removeLast();
@@ -495,7 +503,7 @@ bool KTextEditor::EditorPrivate::eventFilter(QObject *obj, QEvent *event)
     return false; // always continue processing
 }
 
-QList< KateAbstractInputModeFactory *> KTextEditor::EditorPrivate::inputModeFactories()
+QList<KateAbstractInputModeFactory *> KTextEditor::EditorPrivate::inputModeFactories()
 {
     return m_inputModeFactories.values();
 }
@@ -530,3 +538,23 @@ void KTextEditor::EditorPrivate::saveSearchReplaceHistoryModels()
         cg.writeEntry(QStringLiteral("Replace History"), m_replaceHistoryModel->stringList());
     }
 }
+
+KSharedConfigPtr KTextEditor::EditorPrivate::config()
+{
+    // use dummy config for unit tests!
+    if (KTextEditor::EditorPrivate::unitTestMode()) {
+        return KSharedConfig::openConfig(QStringLiteral("katepartrc-unittest"), KConfig::SimpleConfig, QStandardPaths::TempLocation);
+    }
+
+    // else: use application configuration, but try to transfer global settings on first use
+    auto applicationConfig = KSharedConfig::openConfig();
+    if (!KConfigGroup(applicationConfig, QStringLiteral("KTextEditor Editor")).exists()) {
+        auto globalConfig = KSharedConfig::openConfig(QStringLiteral("katepartrc"));
+        for (const auto &group : {QStringLiteral("Editor"), QStringLiteral("Document"), QStringLiteral("View"), QStringLiteral("Renderer")}) {
+            KConfigGroup origin(globalConfig, group);
+            KConfigGroup destination(applicationConfig, QStringLiteral("KTextEditor ") + group);
+            origin.copyTo(&destination);
+        }
+    }
+    return applicationConfig;
+}
